from xml.etree import cElementTree as ctree

class Base(object):

    def addFunction(self, func_code, from_file=''):
        """Add function 
        """
        if from_file:
            with open(from_file) as f:
                func_code = f.read()

        root = self.etree
        t_execute = root.find('execute')

        if not t_execute :
            t_execute = ctree.SubElement(root, 'execute')

        t_execute.text = "\n\t![CDATA]{0:>5}]]\n\t".format(func_code.ljust(4))

        return True

    def removeFunction(self):
        """Remove function tag
        """
        root = self.etree
        t_execute = root.find('execute')
        try:
            root.remove(t_execute)
            return True
        except (Exception,) as e:
            print(e)

        return False


class BaseBinder(Base):
    """Represents any element under <bindings> : <select>,<insert>,<update>,<delete> and <function>
    """
    def __init__(self, name, **kwargs):
        """
        """
        self.name = name
        vars(self).update(kwargs)

        self.etree = self._buildElementTree()

        if vars(self).get('urls',None):
            [ self.addUrl(url) for url in self.urls ]

        if vars(self).get('inputs',None):
            [ self.addInput(elt) for elt in self.inputs ]

        if vars(self).get('paging',None):
            self.addPaging(self.paging)
        
    def _buildElementTree(self,):
        """Turns object into a Element Tree
        """
        t_binder = ctree.Element(self.name)

        for k,v in self.__dict__.items():
            if k not in ('name', 'urls', 'inputs', 'paging') and v :
                t_binder.set(k,v)

        self.etree = t_binder
        return t_binder

    def addUrl(self, url):
        """Add url to binder
        """

        if url not in self.urls:
            self.urls.append(url)

        root = self.etree
        t_urls = root.find('urls')

        if not t_urls:
            t_urls = ctree.SubElement(root, 'urls')

        t_url = ctree.SubElement(t_urls, 'url')
        t_url.text = url

        return True

    def removeUrl(self, url):
        """Remove passed url from a binder
        """

        root = self.etree
        t_urls = root.find('urls')

        if not t_urls:
            return False

        for t_url in t_urls.findall('url'):
            if t_url.text == url.strip():
                t_urls.remove(t_url)
                if url in self.urls:
                    self.urls.remove(url)
                return True

        return False

    def addInput(self, key):
        """Add key to input : key, value or map
        """
        if key not in self.inputs:
            self.inputs.append(key)

        root = self.etree
        t_inputs = root.find('inputs')

        if not t_inputs :
            t_inputs = ctree.SubElement(root, 'inputs')

        t_inputs.append(key.etree)

        return True

    def removeInput(self, key_id, input_type='key'):
        """Remove key (key, value, map) from Input
        key_id : id of the input element i.e <key id='artist' />
        input_type : type of the input ; key, value or map
        """
        root = self.etree
        t_inputs = root.find('inputs')

        if not t_inputs:
            return False

        keys = t_inputs.findall(input_type)

        key = [ key for key in keys if key.get('id') == key_id ]

        try:
            t_inputs.remove(key[0])
            return True
        except (Exception,) as e:
            print(e)

        return False

    def addPaging(self,paging):
        """Add paging to Binder
        """
        if not vars(self).get('paging', None):
            self.paging = paging
        root = self.etree

        try:
            root.append(paging.etree)
            return True
        except (Exception,) as e:
            print(e)

        return False

    def removePaging(self,):
        """Remove paging from Binder
        """
        root = self.etree
        t_paging = root.find('paging')

        try:
            root.remove(t_paging)
            return True
        except (Exception,) as e:
            print(e)

        return False


class BaseInput(object):
    """This class represents an Input Element under Binding element.
    Input Element can be : <key>, <value> or <map>
    Full Documentation https://developer.yahoo.com/yql/guide/yql-opentables-reference.html#yql-opentables-key
    """

    def __init__(self, input_type, id, type, paramType, like='', required=False, default='', private=False, const=False, batchable=False, maxBatchItems=0):
        """
        - input_type : can be <key>, <value> or <map>
        - id : The name of the key. This represents what the user needs to provide in the WHERE clause.
        - like (as):  The alias of the key used in YQL statements.
        - type : The type of data coming back from the Web service.
        - required : A boolean that answers the question
        - paramType : Determines how this key is represented and passed on to the Web service.
        - default : This value is used if one isn't specified by the developer in the SELECT.
        - private : Hide this key's value to the user (in both "desc" and "diagnostics"). 
        - const : A boolean that indicates whether the default attribute must be present and cannot be changed by the end user.
        - batchable : A boolean which answers the question: Does this select and URL support multiple key fetches/requests in a single request (batched fetching)?
        - maxBatchItems : How many requests should be combined in a single batch call. 
        """
        self.name = input_type
        self.id = id
        self.like = like # as is a python <keyword>, that's why in argument we use <like>
        self.type = type
        self.paramType = paramType
        self.required = required
        self.default = default
        self.private = private
        self.const = const
        self.batchable = batchable
        self.maxBatchItems = maxBatchItems

        self.etree = self._buildElementTree()

    def _buildElementTree(self,):
        """Turn object into an ElementTree
        """
        t_elt = ctree.Element(self.name)

        for k,v in [ (key,value) for key,value in self.__dict__.items() if key != 'name']: # Excluding name from list of items
            if v and v != 'false' :
                t_elt.set(k if k != 'like' else 'as', str(v).lower())

        self._etree = t_elt
        return t_elt

class BasePaging(object):
    """Class representing a <paging> element under a <select> element of a OpenTable file description
    """

    def __init__(self, model, *args, **kwargs):

        kwargs['model'] = model
        vars(self).update(kwargs)
        
        self.etree = self._buildElementTree()

    def _buildElementTree(self,):
        """Turn object into an Element Tree
        """

        t_paging = ctree.Element('paging')
        t_paging.set('model', self.model)
        for key in self.__dict__.keys():
            if key != 'model':
                t_tag = ctree.SubElement(t_paging, key)
                for item in self.__dict__[key].items():
                    t_tag.set(*item)

        self.etree = t_paging
        return  t_paging



from  myql.contrib.table.base import BaseInput, BasePaging, BaseBinder

class Binder(BaseBinder):
    """Represent a binder : <select>, <insert>, <update>, <delete> 
    """
    def __init__(self, name, itemPath, produces, pollingFrequencySeconds=0, urls=[], inputs=[], paging=None): 

        super(Binder, self).__init__(name, itemPath=itemPath, produces=produces, pollingFrequencySeconds=pollingFrequencySeconds, urls=urls, inputs=inputs, paging=paging)

        
class BinderFunction(BaseBinder):
    """Represent a function : <function>
    """

    def __init__(self, func_name, func_code='', func_file=None, inputs=[]):
        """
        - func_name : name of the stored procedure
        - func_code : your js code passed through a str
        - func_file : file containing your code
        """
        super(BinderFunction, self).__init__('function', inputs=inputs)
        self.func_name = func_name
        self.etree.set('name', func_name)

        if func_code:
            self.addFunction(func_code)

        if func_file:
            self.addFunction('', from_file=func_file)


class InputKey(BaseInput):
    """Class representing a key of an Input
    """

    def __init__(self, *args, **kwargs):
        super(InputKey, self).__init__('key', *args, **kwargs)
        

class InputValue(BaseInput):
    """Class representing value under an Input
    """
    def __init__(self, *args, **kwargs):
        super(InputValue, self).__init__('value', *args, **kwargs)


class InputMap(BaseInput):
    """Class representing map under an Input
    """
    def __init__(self, *args, **kwargs):
        super(InputMap, self).__init__('map', *args, **kwargs)


class PagingPage(BasePaging):
    """Class representing <paging model='page'>
    """
    def __init__(self, start, pageSize, total):
        super(PagingPage, self).__init__('page', start=start, pageSize=pageSize, total=total)


class PagingOffset(BasePaging):
    """Class representing <paging model='offset' matrix='true'>
    """
    def __init__(self, matrix, start, pageSize, total):
        super(PagingOffset, self).__init__('offset', matrix, start=start, pageSize=pageSize, total=total)
        self.matrix = str(matrix).lower()
        self.etree.set('matrix', self.matrix)
        

class PagingUrl(BasePaging):
    """Class representing <paging model='url'>
    """
    def __init__(self, nextpage):
        super(PagingUrl, self).__init__('url', nextpage=nextpage)
       

class BinderMeta(type):

    INPUT_KEYS = ['name', 'itemPath', 'produces', 'pollingFrequencySeconds', 'urls', 'keys', 'pages']

    def __new__(cls, name, bases, dct):

        if name != 'BinderModel':
            binder_attr = {key: value for (key, value) in dct.items() if key in cls.INPUT_KEYS}
            binder_attr['inputs'] = [ value for value in dct.values() if isinstance(value, BaseInput)]
            paging = [ value for value in dct.values() if isinstance(value, BasePaging)]
            if paging :
                binder_attr['paging'] = paging[0]

            binder = Binder(**binder_attr)
            if dct.get('function',None):
                binder.addFunction(func_code='', from_file=dct['function'])
            dct = { key : value for (key, value) in dct.items() if key in ('__module__', '__metaclass__')}
            dct['binder'] = binder
            # Add KeyException Management
        return super(BinderMeta,cls).__new__(cls, name, bases, dct)


class BinderModel(Binder):
    __metaclass__ = BinderMeta




from __future__ import absolute_import

from tests.tests import TestMYQL
from tests.tests import TestWeather
from tests.tests import TestStockScraper
from tests.tests import TestTable
from tests.tests import TestOAuth
from tests.tests import TestSocial
from tests.tests import TestFilters
from tests.tests import TestRemoteFilters
from tests.tests import TestFuncFilters
from tests.tests import TestPaging
from tests.tests import TestMultiQuery



import os
from xml.dom import minidom
from xml.etree import cElementTree as xtree

from myql.contrib.table.base import Base
from myql.contrib.table.binder import Binder, BinderFunction

class Table(Base):
    """Class representating a YQL Table
    """

    _TAB_ATTR = {'xmlns':'http://query.yahooapis.com/v1/schema/table.xsd', 'securityLevel':'any', 'https':'false'}

    def __init__(self, name, author, apiKeyURL, documentationURL, sampleQuery=[], description=None, table_attr=None, bindings=[]):
        """Initialize the class
        """
        self.name = name
        self.author = author
        self.apiKey = apiKeyURL
        self.documentationURL = documentationURL
        self.description = description
        self.sampleQuery = sampleQuery
        self.table_attr = table_attr
        self.etree = self._init_table_elementTree()
        self.bindings = bindings

        if bindings:
            [ self.addBinder(binder) for binder in bindings ]

    def __repr__(self,):
        return "<Table:{0}>".format(self.name)

    def _xml_pretty_print(self, data):
        """Pretty print xml data
        """
        raw_string = xtree.tostring(data, 'utf-8')
        parsed_string = minidom.parseString(raw_string)
        return parsed_string.toprettyxml(indent='\t')

    def _create_table_xml_file(self, data, fname=None):
        """Creates a xml file of the table
        """
        content = self._xml_pretty_print(data)
        if not fname:
            fname = self.name
        with open(fname+".xml", 'w') as f:
            f.write(content)

    def _init_table_elementTree(self, xml=True, db_table=True):
        """Create a table 
        """
        # <table> tag object
        t_table =  xtree.Element('table')
        # <table xmlns='' securityLevel='' htpps=''>
        if not self.table_attr :
            self.table_attr = self._TAB_ATTR
        for attr in self.table_attr.items() :
            t_table.set(*attr)

        # <meta>
        t_meta = xtree.SubElement(t_table, 'meta')

        # Loop over a sorted key,value of class attributes while ignoring table_attr and name
        for key, value in [(k,v) for k,v in sorted(self.__dict__.items(), key=lambda x: x[0]) if k not in ('table_attr','name') ]:
            if isinstance(value, list): # Works for element like sampleQuery
                for elt in value:
                    t_tag = xtree.SubElement(t_meta, key) # setting attribute name as a tag name
                    t_tag.text = elt # Setting attribute  value as text
            else:
                t_tag = xtree.SubElement(t_meta,key)
                t_tag.text = value
      
        ## <bindings>
        t_bindings = xtree.SubElement(t_table, 'bindings')
        ##

        self.etree = t_table
        return t_table

    def save(self, name=None, path=None):
        """Save file as xml
        """
        if path :
            name = os.path.join(path,name)

        try:
            self._create_table_xml_file(self.etree, name)
        except (Exception,) as e:
            print(e)
            return False

        return True

    def addBinder(self, binder):
        """Adds a binder to the file
        """
        root = self.etree
        bindings = root.find('bindings')
        bindings.append(binder.etree)

        return True

    def removeBinder(self, name):
        """Remove a binder from a table
        """
        root = self.etree
        
        t_bindings = root.find('bindings')
        
        t_binder = t_bindings.find(name)

        if t_binder :
            t_bindings.remove(t_binder)
            return True
        
        return False

    
class TableMeta(type):

    TABLE_KEYS = ['name', 'author', 'apiKeyURL', 'documentationURL', 'sampleQuery','description']

    def __new__(cls, name, bases, dct):
        if name != 'TableModel':
            table_attr = {key: value for (key, value) in dct.items() if key in cls.TABLE_KEYS }
            table_attr['bindings'] = [ value for value in dct.values() if isinstance(value, Binder) or isinstance(value, BinderFunction)]
            table = Table(**table_attr)
            dct = { key : value for (key, value) in dct.items() if key in ('__module__', '__metaclass__')}
            dct['table'] = table

        return super(TableMeta, cls).__new__(cls, name, bases, dct)


class TableModel(Table):
    __metaclass__ = TableMeta


def BinderFrom(cls_binder_meta):
    return cls_binder_meta.binder




from __future__ import absolute_import

import os
import pdb
import logging
import json
import unittest
from xml.dom import minidom
from xml.etree import cElementTree as xtree

from yahoo_oauth import OAuth1

from myql import MYQL, YQL
from myql.errors import NoTableSelectedError
from myql.utils import pretty_xml, pretty_json, prettyfy

from myql.contrib.table import Table
from myql.contrib.table import BaseInput
from myql.contrib.table import Binder, BinderFunction, InputKey, InputValue, PagingPage, PagingUrl, PagingOffset

from myql.contrib.weather import Weather
from myql.contrib.finance.stockscraper import StockRetriever

logging.basicConfig(level=logging.DEBUG,format="[%(asctime)s %(levelname)s] [%(name)s.%(module)s.%(funcName)s] %(message)s \n")


logging.getLogger('mYQL').propagate = False
logging.getLogger('yahoo_oauth').disabled = True
logging.getLogger('requests').setLevel(logging.CRITICAL)

def json_write_data(json_data, filename):
    with open(filename, 'w') as fp:
        json.dump(json_data, fp, indent=4, sort_keys=True, ensure_ascii=False)
        return True
    return False


def json_get_data(filename):
    with open(filename, 'r') as fp:
        json_data = json.load(fp)
    return json_data


class TestMYQL(unittest.TestCase):

    def setUp(self,):
        self.yql = MYQL(format='json',community=True)
        self.insert_result = None

    def tearDown(self):
        pass

    def test_desc(self,):
        response = self.yql.desc('weather.forecast')
        logging.debug(prettyfy(response, 'json'))
        self.assertEqual(response.status_code, 200)

    def test_show_tables(self,):
        yql = MYQL(format='xml', community=False)
        response = yql.show_tables(format='xml')
        logging.debug(prettyfy(response, 'xml'))
        self.assertEqual(response.status_code, 200)

    def test_use(self):
        self.yql.use('http://www.josuebrunel.org/users.xml',name='users')
        response = self.yql.raw_query('select * from users', format='xml')
        self.yql.yql_table_url = None
        logging.debug(pretty_xml(response.content))
        self.assertEqual(response.status_code, 200)

    def test_raw_query(self,):
        response = self.yql.raw_query('select name, woeid from geo.states where place="Congo"')
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_get(self,):
        self.yql.format = 'xml'
        response = self.yql.get('geo.countries', ['name', 'woeid'], 1)
        self.yql.format = 'json'
        logging.debug(pretty_xml(response.content))
        self.assertEqual(response.status_code, 200)

    def test_select(self,):
        response = self.yql.select('geo.countries', ['name', 'code', 'woeid']).where(['name', '=', 'Canada'])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_select_in(self,):
        response = self.yql.select('yahoo.finance.quotes').where(['symbol','in',("YHOO","AAPL","GOOG")])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_select_in_2(self,):
        response = self.yql.select('weather.forecast',['units','atmosphere']).where(['woeid','IN',('select woeid from geo.places(1) where text="Paris"',)])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_raise_exception_select_where_in(self,):
        
        with self.assertRaises(TypeError):
            response = self.yql.select('weather.forecast',['units','atmosphere']).where(['woeid','IN',('select woeid from geo.places(1) where text="Paris"')])

    def test_1_insert(self,):
        response = self.yql.insert('yql.storage.admin',('value',),('http://josuebrunel.org',))
        try:
            logging.debug(pretty_json(response.content))
            data = response.json()['query']['results']['inserted']
            logging.debug(data)
            json_write_data(data,'yql_storage.json')
        except (Exception,) as e:
            logging.error(response.content)
            logging.error(e)
 
        self.assertEqual(response.status_code, 200)

    def test_2_check_insert(self,):
        json_data = json_get_data('yql_storage.json')
        response = self.yql.select('yql.storage').where(['name','=',json_data['select']])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)
       
    def test_3_update(self,):
        json_data = json_get_data('yql_storage.json')
        response = self.yql.update('yql.storage',('value',),('https://josuebrunel.org',)).where(['name','=',json_data['update']])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_4_delete(self,):
        json_data = json_get_data('yql_storage.json')
        response = self.yql.delete('yql.storage').where(['name','=',json_data['update']])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_cross_product(self):
        yql = YQL(format='xml', crossProduct=True)
        response = yql.select('weather.forecast').where(['location', '=', '90210'])
        logging.debug("{0} {1}".format(response.status_code, response.reason))
        self.assertEqual(response.status_code, 200)

    def test_variable_substitution(self,):
        yql = YQL()
        var = {'home': 'Congo'}
        yql.set(var) 

        response = yql.select('geo.states', remote_filter=(5,)).where(['place', '=', '@home'])
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_raise_exception_no_table_selected(self):
        with self.assertRaises(NoTableSelectedError):
            response = self.yql.select(None).where([])

class TestMultiQuery(unittest.TestCase):

    def setUp(self,):
        pass

    def tearDown(self,):
        pass

class TestPaging(unittest.TestCase):

    def setUp(self,):
        self.yql = YQL()

    def tearDown(self,):
        pass

    def test_limit(self,):
        data = self.yql.select('geo.states', limit=3, offset=2).where(['place', '=', 'Congo'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
        self.assertEqual(data.json()['query']['count'], 3)

    def test_offset_raw_query(self,):
        data = self.yql.raw_query("SELECT * FROM geo.counties WHERE place='CA' LIMIT 10 OFFSET 5 | sort(field='name')")
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_offset_get(self,):
        data = self.yql.get('yql.table.list', limit=10, offset=3)
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_offset_select(self):
        data = self.yql.select('geo.counties', limit=10, offset=3).where(['place', '=', 'CA'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)


class TestFilters(unittest.TestCase):

    def setUp(self,):
        self.yql = YQL()

    def tearDown(self,):
        pass

    def test_filter_not_equal(self,):
        data = self.yql.select('geo.countries', ['name', 'placeTypeName']).where(['name', 'like', 'A%'], ['placeTypeName.content', '!=', 'Country'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_greater_than_or_equal(self,):
        data = self.yql.select('geo.countries',['woeid', 'name', 'placeTypeName']).where(['place', '=', 'North America'], ['place.woeid', '>=', 56042304])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_less_than_or_equal(self,):
        data = self.yql.select('geo.countries',['name', 'placeTypeName']).where(['place', '=', 'North America'], ['place.woeid', '<=', 23424758])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_not_in(self,):
        data = self.yql.select('geo.countries',['name', 'placeTypeName']).where(['placeTypeName.content', 'NOT IN', ('Country','Territory','Overseas Collectivity')])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_is_null(self,):
        data = self.yql.select('youtube.user').where(['id','=','120u12a'], ['user.description','IS NULL'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_is_not_null(self,):
        data = self.yql.select('youtube.user').where(['id','=','120u12a'], ['user.description','IS NOT NULL'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_like(self,):
        data = self.yql.select('yql.table.list').where(['content','like','%apple%'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_not_like(self,):
        data = self.yql.select('geo.counties', ['name', 'placeTypeName']).where(['place', '=', 'CT'], ['name', 'NOT LIKE', '%d'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_matches(self,):
        data = self.yql.select('yql.table.list').where(['content','MATCHES','.*itunes$'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_filter_not_matches(self,):
        data = self.yql.select('geo.countries', ['name', 'placeTypeName']).where(['placeTypeName.content','NOT MATCHES','^(Country|Territory)$'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)


class TestFuncFilters(unittest.TestCase):

    def setUp(self,):
        self.yql = YQL()

    def tearDown(self,):
        pass

    def test_func_filter_reverse(self,):
        func_filters = ['reverse']
        data = self.yql.select('geo.states', func_filters=func_filters).where(['place', '=', 'Congo'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_func_filter_tail(self,):
        func_filters = [('tail', 2)]
        data = self.yql.select('geo.states', func_filters=func_filters).where(['place', '=', 'Congo'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_func_filter_truncate(self,):
        func_filters = [('truncate', 2)]
        data = self.yql.select('geo.states', func_filters=func_filters).where(['place', '=', 'Congo'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_func_filter_sanitize(self,):
        #func_filters = [('sanitize', '')]
        #data = self.yql.select('geo.states', func_filters=func_filters).where(['place', '=', 'Congo'])
        #logging.debug(pretty_json(data.content))
        #self.assertEqual(data.status_code, 200)
        pass

    def test_func_filter_sort(self,):
        func_filters = [
            {'sort': [
                ('field','name'),
                ('descending','true')
            ]},
            ('tail', 10),
            #('reverse')
        ]
        data = self.yql.select('geo.counties', func_filters=func_filters).where(['place', '=', 'CA'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_func_filter_unique(self,):
        func_filters = [
            {'unique': [
                ('field','content'),
                ('hideRepeatCount','false')
            ]},
            ('truncate', 5)
        ]
        data = self.yql.get('yql.table.list', func_filters=func_filters)
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_raise_exception_func_filter(self):
        func_filters = 'unique'
        with self.assertRaises(TypeError):
           data =  self.yql.get('yql.table.list', func_filters=func_filters)

    def test_raise_exception_func_filter_invalid_type(self):
        func_filters = [30]
        with self.assertRaises(TypeError):
            data = self.yql.get('yql.table.list', func_filters=func_filters)


class TestRemoteFilters(unittest.TestCase):

    def setUp(self,):
        self.yql = YQL(diagnostics=True, )

    def test_remote_filter_get_count(self,):
        data = self.yql.get('geo.countries', remote_filter=(10,))
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_remote_filter_get_start_and_count(self,):
        data = self.yql.get('geo.countries', remote_filter=(100,10))
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
        

    def test_remote_filter_select_count(self,):
        data = self.yql.select('geo.counties', remote_filter=(20,)).where(['place', '=', 'CA'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
        
    def test_remote_filter_select_start_and_count(self,):
        data = self.yql.select('geo.counties', remote_filter=(60,20)).where(['place', '=', 'CA'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_raise_exception_remote_filter_not_tuple(self,):

        with self.assertRaises(TypeError):
            data = self.yql.get('geo.countries', remote_filter=10)


class TestOAuth(unittest.TestCase):

    def setUp(self,):
        pass

    def tearUp(self,):
        pass

    def test_get_guid(self,):
        oauth = OAuth1(None, None, from_file='credentials.json')
        yql = MYQL(format='json', oauth=oauth)
        response = yql.get_guid('josue_brunel')
        logging.debug(pretty_json(response.content))
        self.assertEqual(response.status_code, 200)

    def test_yahoo_fantasy_sport(self,):
        oauth = OAuth1(None, None, from_file='credentials.json')
        yql = MYQL(format='json', oauth=oauth)
        teams = ('mlb.l.1328.t.1','mlb.l.1328.t.2')
        year = '2015-05-05'
        for team in teams:
            response = yql.select('fantasysports.teams.roster').where(['team_key','=',team],['date','=',year])
            self.assertEqual(response.status_code, 200)
            if not response.status_code == 200:
                return False

            data = response.json()
            try:
                current_team = data['query']['results']['team']
                print(current_team['team_id'],current_team['name'],current_team['number_of_trades'],current_team['number_of_moves'])
            except (Exception,) as e:
                print(e)


class TestWeather(unittest.TestCase):
    """Weather module unit test
    """
    def setUp(self,):
        self.weather = Weather(unit='c', format='json')

    def test_get_weather_in(self):
        data = self.weather.get_weather_in('choisy-le-roi')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_weather_in_with_unit(self):
        data = self.weather.get_weather_in('choisy-le-roi', 'c',['location', 'units', 'item.condition'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
    
    def test_get_weather_forecast(self,):
        data = self.weather.get_weather_forecast('choisy-le-roi')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
    
    def test_get_weather_description(self,):
        data = self.weather.get_weather_description('dolisie')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_current_condition(self,):
        data = self.weather.get_current_condition('Nantes')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_current_atmosphere(self,):
        data = self.weather.get_current_atmosphere('Scotland')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_current_wind(self,):
        data = self.weather.get_current_wind('Barcelona')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
 
    def test_get_astronomy(self,):
        data = self.weather.get_astronomy('Congo')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
 

class TestStockScraper(unittest.TestCase):

    def setUp(self,):
        self.stock = StockRetriever(format='json')

    def tearDown(self):
        pass

    def test_get_current_info(self,):
        data = self.stock.get_current_info(["YHOO","AAPL","GOOG","J&KBANK.BO"])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)
        
    def test_get_current_info_with_one_symbol(self,):
        data = self.stock.get_current_info(["J&KBANK.BO"])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_news_feed(self,):
        data = self.stock.get_news_feed('YHOO')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_historical_info_with_args(self,):
        data = self.stock.get_historical_info('YHOO',items=['Open','Close','High','Low'], limit=5,startDate='2014-09-11',endDate='2015-02-10')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200) 

    def test_get_historical_info_without_args(self,):
        data = self.stock.get_historical_info('YHOO')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_options_info(self,):
        data = self.stock.get_options_info('YHOO')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)   

    def test_get_index_summary(self,):
        data = self.stock.get_index_summary('GOOG',('Volume','Change'))
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)   

    def test_get_industry_index(self,):
        data = self.stock.get_industry_index(112)
        #logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)   

    def test_get_symbols(self,):
        data = self.stock.get_symbols('Google')
        logging.debug(pretty_json(data.content))
        self.assertIn(data.status_code, (200, 400, "Web Service is currently Down!!"))  

    def test_get_xchange_rate(self,):
        data = self.stock.get_xchange_rate(['EURUSD','GBPUSD'])
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_dividendhistory(self,):
        data = self.stock.get_dividendhistory('AAPL',"2008-01-01", "2015-06-15")
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

    def test_get_balancesheet(self,):
        data = self.stock.get_balancesheet('YHOO')
        logging.debug(pretty_json(data.content))
        self.assertEqual(data.status_code, 200)

class TestSocial(unittest.TestCase):

    def setUp(self,):
        self.oauth = OAuth1(None, None, from_file='credentials.json')
        self.yql = YQL(oauth=self.oauth)

    #def test_get_contacts(self,):
    #    data = self.yql.select('social.contacts').where(['guid','=', '@me'])
    #    logging.debug(pretty_json(data.content))
    #    self.assertEqual(data.status_code, 200)

class TestTable(unittest.TestCase):

    def setUp(self,):
        self.table_desc = {
            'name': 'mytest',
            'author': 'josuebrunel',
            'apiKeyURL': 'http://josuebrunel.org/api',
            'documentationURL': 'http://josuebrunel.org/doc.html',
            'sampleQuery': ['SELECT * FROM mytable', 'SELECT name FROM mytable WHERE id="345"','DELETE FROM mytable WHERE id="345"'],
        }

        self.table = Table(**self.table_desc)

        self.binder_desc = {
            'name': 'select',
            'itemPath': 'products.product',
            'produces': 'xml'
        }

        self.binder = Binder(**self.binder_desc)
        self.binder_insert = Binder('insert','products.product','json')

        self.key_desc = {
            'id': 'artist',
            'type': 'xs:string',
            'paramType': 'path'
        }

        self.key = InputKey(**self.key_desc)
        self.key2 = InputKey(id='song', type='xs:string', paramType='path', required='true')

        start= {'id': 'ItemPage', 'default': '1'}
        pageSize= {'id':'Count' ,'max':'25'}
        total= {'default': '10'}
        self.paging = PagingPage(start, pageSize, total)

    def xml_pretty_print(self, data):
        """Pretty logging.debug xml data
        """
        raw_string = xtree.tostring(data, 'utf-8')
        parsed_string = minidom.parseString(raw_string)
        return parsed_string.toprettyxml(indent='\t')

    def test_add_binder(self,):
        self.assertEqual(self.table.addBinder(self.binder),True)
        logging.debug(self.xml_pretty_print(self.table.etree))

    def test_remove_binder(self,):
        self.binder.addInput(self.key)
        self.binder_insert.addInput(self.key)
        self.binder.addFunction('', from_file='tests_data/jscode.js')
        self.binder_insert.addFunction("console.log('hello this is an insert function'); ")
        self.table.addBinder(self.binder)
        self.table.addBinder(self.binder_insert)
        self.table.save(name='before', path='tests_data')
        self.assertEqual(os.path.isfile('tests_data/before.xml'),True)
        self.table.removeBinder('select')
        self.table.save(name='after', path='tests_data')
        self.assertEqual(os.path.isfile('tests_data/after.xml'),True)

    def test_add_input_to_binder(self,):
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addInput(self.key),True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_remove_input(self,):
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addInput(self.key),True)
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addInput(self.key2),True)
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.removeInput(key_id='artist'),True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_add_function_from_file(self,):
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addFunction('', from_file='tests_data/jscode.js'),True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_remove_function(self,):
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addFunction('', from_file='tests_data/jscode.js'),True)
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.removeFunction(), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_add_paging(self,):
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addPaging(self.paging), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_create_binder_with_paging(self,):
        start= {'id': 'ItemPage', 'default': '1'}
        pageSize= {'id':'Count' ,'max':'25'}
        total= {'default': '10'}
        paging = PagingPage(start, pageSize, total)
        logging.debug(self.xml_pretty_print(paging.etree))
        self.binder_desc['paging']=paging
        logging.debug(self.binder_desc)
        binder = Binder(**self.binder_desc)
        self.assertNotEqual(binder.paging,None)
        logging.debug(self.xml_pretty_print(binder.etree))

    def test_create_binder_with_offset_paging(self,):
        start= {'id': 'ItemPage', 'default': '1'}
        pageSize= {'id':'Count' ,'max':'25'}
        total= {'default': '10'}
        paging = PagingOffset(True,  start, pageSize, total)
        logging.debug(self.xml_pretty_print(paging.etree))
        self.binder_desc['paging']=paging
        logging.debug(self.binder_desc)
        binder = Binder(**self.binder_desc)
        self.assertNotEqual(binder.paging,None)
        logging.debug(self.xml_pretty_print(binder.etree))


    def test_create_binder_with_url_paging(self,):
        nextpage = {'path': 'ysearchresponse.nextpage'}
        paging = PagingUrl(nextpage)
        logging.debug(self.xml_pretty_print(paging.etree))
        self.binder_desc['paging']=paging
        logging.debug(self.binder_desc)
        binder = Binder(**self.binder_desc)
        self.assertNotEqual(binder.paging,None)
        logging.debug(self.xml_pretty_print(binder.etree))


    def test_remove_paging(self,):
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addPaging(self.paging), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.removePaging(), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_create_binder_with_urls(self,):
        url = 'http://josuebrunel.org/service/v1'
        url2 = 'http://josuebrunel.org/service/v1/?name=lol'
        self.binder_desc['urls'] = [url, url2]
        binder = Binder(**self.binder_desc)
        logging.debug(self.xml_pretty_print(binder.etree))
        self.assertEqual(self.binder.addUrl(url), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))


    def test_add_url(self,):
        url = 'http://josuebrunel.org/service.js'
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addUrl(url), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_remove_url(self,):
        url = 'http://josuebrunel.org/service.js'
        url2 = 'http://google.com'
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addUrl(url), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.addUrl(url2), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))
        self.assertEqual(self.binder.removeUrl(url), True)
        logging.debug(self.xml_pretty_print(self.binder.etree))

    def test_save_file(self,):
        self.table.save()
        self.assertEqual(os.path.isfile('mytest.xml'),True) 

    def test_save_with_another_name(self):
        name = "tests_data/toto"
        self.table.save(name)
        self.assertEqual(os.path.isfile(name+'.xml'),True)

    def test_save_to_different_location(self,):
        fname = "titi"
        path = 'tests_data'
        name = os.path.join(path,fname)
        self.table.save(name=fname, path=path)
        self.assertEqual(os.path.isfile(name+'.xml'),True)

    def test_create_table(self,):
        self.binder.addInput(self.key)
        self.binder.addFunction('', from_file='tests_data/jscode.js')
        logging.debug(self.xml_pretty_print(self.table.etree))
        self.table.addBinder(self.binder)
        self.table.save(name='mytable', path='tests_data')
        self.assertEqual(os.path.isfile('tests_data/mytable.xml'),True)
        logging.debug(self.xml_pretty_print(self.table.etree))

    def test_create_table_and_add_two_binders(self,):
        self.binder.addInput(self.key)
        self.binder_insert.addInput(self.key)
        self.binder.addFunction('', from_file='tests_data/jscode.js')
        self.binder_insert.addFunction("console.log('hello this is an insert function'); ")
        logging.debug(self.xml_pretty_print(self.table.etree))
        self.table.addBinder(self.binder)
        logging.debug(self.xml_pretty_print(self.table.etree))
        self.table.addBinder(self.binder_insert)
        self.table.save(name='mytable', path='tests_data')
        self.assertEqual(os.path.isfile('tests_data/mytable.xml'),True)
        logging.debug(self.xml_pretty_print(self.table.etree))

    def test_create_table_with_binder(self,):
        self.binder.addInput(self.key)
        self.binder.addFunction('', from_file='tests_data/jscode.js')
        self.table_desc['bindings'] = [self.binder]
        table = Table(**self.table_desc)
        logging.debug(self.xml_pretty_print(table.etree))
        table.save(name='mytable', path='tests_data')
        self.assertEqual(os.path.isfile('tests_data/mytable.xml'),True)
        logging.debug(self.xml_pretty_print(table.etree))

    def test_create_table_with_two_binders(self,):
        self.binder.addInput(self.key)
        self.binder.addFunction('', from_file='tests_data/jscode.js')
        self.table_desc['bindings'] = [self.binder, self.binder_insert]
        table = Table(**self.table_desc)
        logging.debug(self.xml_pretty_print(table.etree))
        table.save(name='mytable', path='tests_data')
        self.assertEqual(os.path.isfile('tests_data/mytable.xml'),True)
        logging.debug(self.xml_pretty_print(table.etree))

    def test_add_function_table(self):
        logging.debug(self.xml_pretty_print(self.table.etree))
        bf = BinderFunction('concat', inputs=[self.key, self.key2])
        bf.addFunction('', from_file='tests_data/jscode.js')
        self.table.addBinder(bf)
        #self.assertEqual(self.table.addFunction('', from_file='tests_data/jscode.js'),True)
        logging.debug(self.xml_pretty_print(self.table.etree))

    def test_create_function_with_func_code(self):
        logging.debug(self.xml_pretty_print(self.table.etree))
        bf = BinderFunction('concat', func_code='console.log("hello moron !!!")')
        logging.debug(self.xml_pretty_print(bf.etree))

    def test_create_function_with_func_file(self):
        logging.debug(self.xml_pretty_print(self.table.etree))
        bf = BinderFunction('concat', func_file='tests_data/jscode.js')
        logging.debug(self.xml_pretty_print(bf.etree))


    def test_remove_function_table(self,):
        logging.debug(self.xml_pretty_print(self.table.etree))
        self.assertEqual(self.table.addFunction('', from_file='tests_data/jscode.js'),True)
        logging.debug(self.xml_pretty_print(self.table.etree))
        self.assertEqual(self.table.removeFunction(),True)
        logging.debug(self.xml_pretty_print(self.table.etree))

    def test_baseinput_to_xml(self,):
        i = BaseInput('key','name','xs:string', 'path', required=True, default='josh', private=True, maxBatchItems=10)
        logging.debug(self.xml_pretty_print(i.etree))

    def test_inputvalue(self,):
        v = InputValue('content', 'xs:string', 'variable', required=True)
        logging.debug(self.xml_pretty_print(v.etree))
    
    def tearUp(self):
        os.path.unlink('tests_data/mytest.xml')
        os.path.unlink('tests_data/toto.xml')
        
if '__main__' == __name__:
    unittest.main()




from __future__ import absolute_import

from myql.myql import YQL


class Weather(YQL):
    """Weather Class
    """

    def __init__(self, unit=None, **kwargs):
        """Initialize a weather object
        """
        kwargs.update({'community': False})

        super(Weather, self).__init__(**kwargs)

        self.unit = unit 

    def get_weather_in(self, place, unit=None, items=None):
        """Return weather info according to place
        """
        unit = unit if unit else self.unit
        response = self.select('weather.forecast', items=items).where(['woeid','IN',('SELECT woeid FROM geo.places WHERE text="{0}"'.format(place),)], ['u','=',unit] if unit else [])
        return response

    def get_weather_forecast(self, place, unit=None):
        """Return weather forecast accoriding to place
        """
        unit = unit if unit else self.unit
        response = self.get_weather_in(place, items=['item.forecast'], unit=unit)
        return response        

    def get_weather_description(self, place):
        """Return weather description 
        """
        response = self.get_weather_in(place, items=['item.condition.text'])
        return response

    def get_current_condition(self, place):
        """Return weather condition
        """
        response = self.get_weather_in(place, items=['item.condition'])
        return response

    def get_current_atmosphere(self, place):
        """Return weather atmosphere
        """
        response = self.get_weather_in(place, items=['atmosphere'])
        return response

    def get_current_wind(self, place):
        """Return weather wind  
        """
        response = self.get_weather_in(place, items=['wind'])
        return response

    def get_astronomy(self, place):
        """Return sunrise and sunset time
        """
        response = self.get_weather_in(place, items=['astronomy'])
        return response



"""Simple Python Wrapper of the Yahoo! Query Language
"""

from __future__ import absolute_import

import re
import logging

import requests
from myql import errors 


logging.basicConfig(level=logging.DEBUG,format="[%(asctime)s %(levelname)s] [%(name)s.%(module)s.%(funcName)s] %(message)s \n")
logger = logging.getLogger('mYQL')

logging.getLogger('requests').disabled = True # Disabling requests default logger


class YQL(object):
    '''Yet another Python Yahoo! Query Language Wrapper
    Attributes:
    - url : data provider url
    - table : default table, so you won't have to specify a table later
    - format : default format of the responses
    - diagnostics : set to <True> to see diagnostics on queries
    - community : set to <True> to have access to community tables
    '''
    PUBLIC_URL = 'https://query.yahooapis.com/v1/public/yql'
    PRIVATE_URL = 'https://query.yahooapis.com/v1/yql'
    COMMUNITY_DATA  = "env 'store://datatables.org/alltableswithkeys'; " #Access to community table 

    FUNC_FILTERS = ['sort', 'tail', 'truncate', 'reverse', 'unique', 'sanitize']
  
    def __init__(self, community=True, format='json', jsonCompact=True, crossProduct=None, debug=False, diagnostics=False, oauth=None):
        self.community = community # True means access to community data
        self.format = format
        self._table = None
        self._query = None # used to build query when using methods such as <select>, <insert>, ...
        self._payload = {} # Last payload
        self._vars = {}
        self.diagnostics = diagnostics # Who knows, someone would like to turn it ON lol
        self._limit = None
        self._offset = None
        self.crossProduct = crossProduct
        self.jsonCompact = jsonCompact
        self.debug = debug
    
        if oauth:
            self.oauth = oauth

    def __repr__(self):
        '''Returns information on the current instance
        '''
        return "<Community>: {0} - <Format>: {1} ".format(self.community, self.format)

    def _payload_builder(self, query, format=None):
        '''Build the payload'''
        if self.community :
            query = self.COMMUNITY_DATA + query # access to community data tables

        if vars(self).get('yql_table_url') : # Attribute only defined when MYQL.use has been called before
            query = "use '{0}' as {1}; ".format(self.yql_table_url, self.yql_table_name) + query

        if vars(self).get('_func'): # if post query function filters
            query = '| '.join((query, self._func))

        self._query = query        
        
        self._query = self._add_limit()
        self._query = self._add_offset()

        logger.info("QUERY = %s" %(self._query,))

        payload = {
            'q': self._query,
            'callback': '',#This is not javascript
            'diagnostics': self.diagnostics,
            'format': format if format else self.format,
            'debug': self.debug,
            'jsonCompact': 'new' if self.jsonCompact else ''
        }

        if vars(self).get('_vars'):
            payload.update(self._vars)

        if self.crossProduct:
            payload['crossProduct'] = 'optimized'

        self._payload = payload
        logger.info("PAYLOAD = %s " %(payload, ))

        return payload

    def raw_query(self, query, format=None, pretty=False):
        '''Executes a YQL query and returns a response
        >>>...
        >>> resp = yql.raw_query('select * from weather.forecast where woeid=2502265')
        >>>
        '''
        if format:
            format = format
        else:
            format = self.format

        payload = self._payload_builder(query, format=format)
        response = self.execute_query(payload)
        if pretty:
            response = self.response_builder(response)

        return response

    def execute_query(self, payload):
        '''Execute the query and returns and response'''
        if vars(self).get('oauth'):
            if not self.oauth.token_is_valid(): # Refresh token if token has expired
                self.oauth.refresh_token()
            response = self.oauth.session.get(self.PRIVATE_URL, params= payload, header_auth=True)
        else:
            response = requests.get(self.PUBLIC_URL, params= payload)

        self._response = response # Saving last response object.
        return response

    def _clause_formatter(self, cond):
        '''Formats conditions
        args is a list of ['field', 'operator', 'value']
        '''

        if len(cond) == 2 :
            cond = ' '.join(cond)
            return cond

        if 'in' in cond[1].lower() :
            if not isinstance(cond[2], (tuple, list)):
                    raise TypeError('("{0}") must be of type <type tuple> or <type list>'.format(cond[2]))

            if 'select' not in cond[2][0].lower() :
                cond[2] = "({0})".format(','.join(map(str,["'{0}'".format(e) for e in cond[2]])))
            else:
                cond[2] = "({0})".format(','.join(map(str,["{0}".format(e) for e in cond[2]])))

            cond = " ".join(cond)
        else: 
            #if isinstance(cond[2], str):
            #    var = re.match('^@(\w+)$', cond[2])
            #else:
            #    var = None
            #if var :
            if isinstance(cond[2], str) and cond[2].startswith('@'):
                cond[2] = "{0}".format(cond[2])
            else :
                cond[2] = "'{0}'".format(cond[2])
            cond = ' '.join(cond)

        return cond
    
    def response_builder(self, response):
        '''Try to return a pretty formatted response object
        '''
        try:
            r = response.json()
            result = r['query']['results']
            response = {
                'num_result': r['query']['count'] ,
                'result': result
            }
        except (Exception,) as e:
            print(e)
            return response.content

        return response

    def _func_filters(self, filters):
        '''Build post query filters
        '''
        if not isinstance(filters, (list,tuple)):
            raise TypeError('func_filters must be a <type list> or <type tuple>')

        for i, func in enumerate(filters) :
            if isinstance(func, str) and func == 'reverse':
                filters[i] = 'reverse()'
            elif isinstance(func, tuple) and func[0] in YQL.FUNC_FILTERS:
                filters[i] = '{:s}(count={:d})'.format(*func)
            elif isinstance(func, dict) :
                func_stmt = ''
                func_name = list(func.keys())[0] # Because of Py3
                values = [ "{0}='{1}'".format(v[0], v[1]) for v in func[func_name] ]
                func_stmt = ','.join(values)
                func_stmt = '{0}({1})'.format(func_name, func_stmt)
                filters[i] = func_stmt
            else:
                raise TypeError('{0} is neither a <str>, a <tuple> or a <dict>'.format(func))
        return '| '.join(filters) 

    def _add_limit(self,):
        return ''.join((self._query," LIMIT {0} ".format(self._limit))) if self._limit else self._query

    def _add_offset(self,): 
        return ''.join((self._query," OFFSET {0} ".format(self._offset))) if self._offset else self._query


    ######################################################
    #
    #                 MAIN METHODS
    #
    #####################################################

    ##USE
    def use(self, url, name='mytable'):
        '''Changes the data provider
        >>> yql.use('http://myserver.com/mytables.xml')
        '''
        self.yql_table_url = url
        self.yql_table_name = name
        return {'table url': url, 'table name': name}

    ##SET
    def set(self, myvars):
        '''Define variable to be substitute in the query
        '''
        self._vars.update(myvars)
        return True

    ##DESC
    def desc(self, table):
        '''Returns table description
        >>> yql.desc('geo.countries')
        >>>
        '''
        query = "desc {0}".format(table)
        response = self.raw_query(query)

        return response

    ##GET
    def get(self, *args, **kwargs):
        '''Just a select which returns a response
        >>> yql.get("geo.countries', ['name', 'woeid'], 5")
        '''
        self = self.select(*args, **kwargs)

        payload = self._payload_builder(self._query)
        response = self.execute_query(payload)

        return response
    
    ## SELECT
    def select(self, table, items=None, limit=None, offset=None, remote_filter=None, func_filters=None):
        '''This method simulate a select on a table
        >>> yql.select('geo.countries', limit=5) 
        >>> yql.select('social.profile', ['guid', 'givenName', 'gender'])
        '''
        self._table = table

        if remote_filter:
            if not isinstance(remote_filter, tuple):
                raise TypeError("{0} must be of type <type tuple>".format(remote_filter))

            table = "%s(%s)" %(table, ','.join(map(str, remote_filter)))

        if not items:
            items = ['*']
        self._query = "SELECT {1} FROM {0} ".format(table, ','.join(items))

        if func_filters:
            self._func = self._func_filters(func_filters) 

        self._limit = limit
        self._offset = offset
            
        return self

    ## MULTI QUERY
    def multi_query(self, queries):
        """Allow multi query
        """
        pass

    ## INSERT
    def insert(self, table,items, values):
        """This method allows to insert data into table
        >>> yql.insert('bi.ly.shorten',('login','apiKey','longUrl'),('YOUR LOGIN','YOUR API KEY','YOUR LONG URL'))
        """
        values = ["'{0}'".format(e) for e in values]
        self._query = "INSERT INTO {0} ({1}) VALUES ({2})".format(table,','.join(items),','.join(values))
        payload = self._payload_builder(self._query)
        response = self.execute_query(payload)

        return response

    ## UPDATE
    def update(self, table, items, values):
        """Updates a YQL Table
        >>> yql.update('yql.storage',['value'],['https://josuebrunel.orkg']).where(['name','=','store://YEl70PraLLMSMuYAauqNc7']) 
        """
        self._table = table
        self._limit = None
        items_values = ','.join(["{0} = '{1}'".format(k,v) for k,v in zip(items,values)])
        self._query = "UPDATE {0} SET {1}".format(self._table, items_values)

        return self

    ## DELETE
    def delete(self, table):
        """Deletes record in table
        >>> yql.delete('yql.storage').where(['name','=','store://YEl70PraLLMSMuYAauqNc7'])
        """
        self._table = table
        self._limit = None
        self._query = "DELETE FROM {0}".format(self._table)

        return self

    ## WHERE
    def where(self, *args):
        ''' This method simulates a where condition. Use as follow:
        >>> yql.select('mytable').where(['name', '=', 'alain'], ['location', '!=', 'paris'])
        '''
        if not self._table:
            raise errors.NoTableSelectedError('No Table Selected')

        clause = []
        self._query += ' WHERE '

        clause = [ self._clause_formatter(x) for x in args if x ]

        self._query += ' AND '.join(clause)

        payload = self._payload_builder(self._query)
        response = self.execute_query(payload)

        return response


class MYQL(YQL):

    def __init__(self, *args, **kwargs):

        super(MYQL, self).__init__(**kwargs) 


    ######################################################
    #
    #                     HELPERS
    #
    #####################################################

    def get_guid(self, username):
        '''Returns the guid of the username provided
        >>> guid = self.get_guid('josue_brunel')
        >>> guid
        '''
        response = self.select('yahoo.identity').where(['yid', '=', username])
    
        return response
    
    def show_tables(self, format='json'):
        '''Return list of all available tables'''

        query = 'SHOW TABLES'
        payload = self._payload_builder(query, format) 	

        response = self.execute_query(payload) 

        return response




class NoTableSelectedError(Exception):
    '''Error raised when no table has been selected
    '''
    def __init__(self, msg=None):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)
 



"""This module isn't mine, it's at 99% inspired from https://github.com/gurch101/StockScraper written by Gurchet Rai.
So all rigts reserved to Gurchet Rai.
Documentation http://www.gurchet-rai.net/dev/yahoo-finance-yql
"""

from __future__ import absolute_import

import re
import json
from datetime import date, timedelta

import requests

from myql.myql import YQL

class StockRetriever(YQL):

    def __init__(self, format='json', debug=False, oauth=None):
        """Initialize the object
        """
        super(StockRetriever, self).__init__(community=True, format=format, debug=debug, oauth=oauth)
    
    def __get_time_range(self, startDate, endDate):
        """Return time range
        """
        today = date.today()
        start_date = today - timedelta(days=today.weekday(), weeks=1)
        end_date = start_date + timedelta(days=4)

        startDate = startDate if startDate else str(start_date)
        endDate = endDate if endDate else str(end_date)
        
        return startDate, endDate

    def get_current_info(self, symbolList, columns=None):
        """get_current_info() uses the yahoo.finance.quotes datatable to get all of the stock information presented in the main table on a typical stock page 
        and a bunch of data from the key statistics page.
        """
        response = self.select('yahoo.finance.quotes',columns).where(['symbol','in',symbolList])
        return response

    def get_news_feed(self, symbol):
        """get_news_feed() uses the rss data table to get rss feeds under the Headlines and Financial Blogs headings on a typical stock page.
        """
        rss_url='http://finance.yahoo.com/rss/headline?s={0}'.format(symbol)
        response = self.select('rss',['title','link','description'],limit=2).where(['url','=',rss_url])
        return response

    def get_historical_info(self, symbol,items=None, startDate=None, endDate=None, limit=None):
        """get_historical_info() uses the csv datatable to retrieve all available historical data on a typical historical prices page
        """
        startDate, endDate = self.__get_time_range(startDate, endDate)
        response = self.select('yahoo.finance.historicaldata',items,limit).where(['symbol','=',symbol],['startDate','=',startDate],['endDate','=',endDate])
        return response

    def get_options_info(self, symbol, items=None, expiration=''):
        """get_options_data() uses the yahoo.finance.options table to retrieve call and put options from the options page.
        """
        response = self.select('yahoo.finance.options',items).where(['symbol','=',symbol],[] if not expiration else ['expiration','=',expiration])
        return response

    def get_index_summary(self, symbol, items=None):
        """
        """
        response = self.select('yahoo.finance.quoteslist',items).where(['symbol','=',symbol])
        return response

    def get_industry_index(self, index_id,items=None):
        """retrieves all symbols that belong to an industry.
        """
        response = self.select('yahoo.finance.industry',items).where(['id','=',index_id])
        return response

    def get_xchange_rate(self, pairs, items=None):
        """Retrieves currency exchange rate data for given pair(s). 
        Accepts both where pair='eurusd, gbpusd' and where pair in ('eurusd', 'gpbusd, usdaud')
        """
        response = self.select('yahoo.finance.xchange', items).where(['pair', 'in', pairs])
        return response

    def get_dividendhistory(self, symbol, startDate, endDate, items=None):
        """Retrieves divident history
        """
        startDate, endDate = self.__get_time_range(startDate, endDate)
        response = self.select('yahoo.finance.dividendhistory', items).where(['symbol', '=', symbol], ['startDate', '=', startDate], ['endDate', '=', endDate])
        return response

    def get_balancesheet(self, symbol):
        """Retrieves balance sheet
        """
        response = self.select('yahoo.finance.balancesheet').where(['symbol', '=', symbol])
        return response

    def get_symbols(self, name):
        """Retrieves all symbols belonging to a company
        """
        url = "http://autoc.finance.yahoo.com/autoc?query={0}&callback=YAHOO.Finance.SymbolSuggest.ssCallback".format(name)

        response = requests.get(url)

        json_data = re.match("YAHOO\.Finance\.SymbolSuggest.ssCallback\((.*)\)", response.text)
        try:
            json_data = json_data.groups()[0]
        except (Exception,) as e:
            print(e)
            json_data = '{"results": "Webservice seems to be down"}'

        return type('response', (requests.Response,),{
            'text' : json_data,
            'content': json_data.encode(),
            'status_code': response.status_code,
            'reason': response.reason,
            'encoding': response.encoding,
            'apparent_encoding': response.apparent_encoding,
            'cookies': response.cookies,
            'headers': response.headers,
            'json': lambda : json.loads(json_data),
            'url': response.url
        })



import json
from xml.dom import minidom


def pretty_json(data):
    """Return a pretty formatted json
    """
    data = json.loads(data.decode('utf-8'))
    return json.dumps(data, indent=4, sort_keys=True)


def pretty_xml(data):
    """Return a pretty formated xml
    """
    parsed_string = minidom.parseString(data.decode('utf-8'))
    return parsed_string.toprettyxml(indent='\t', encoding='utf-8')


def prettyfy(response, format='json'):
    """A wrapper for pretty_json and pretty_xml
    """
    if format == 'json':
        return pretty_json(response.content)
    else:
        return pretty_xml(response.content)

